This is a [Next.js](https://nextjs.org) project bootstrapped with [`create-next-app`](https://nextjs.org/docs/app/api-reference/cli/create-next-app).

## Getting Started

First, run the development server:

```bash
npm run dev
# or
yarn dev
# or
pnpm dev
# or
bun dev
```

Open [http://localhost:3000](http://localhost:3000) with your browser to see the result.

You can start editing the page by modifying `app/page.tsx`. The page auto-updates as you edit the file.

This project uses [`next/font`](https://nextjs.org/docs/app/building-your-application/optimizing/fonts) to automatically optimize and load [Geist](https://vercel.com/font), a new font family for Vercel.

## Learn More

To learn more about Next.js, take a look at the following resources:

- [Next.js Documentation](https://nextjs.org/docs) - learn about Next.js features and API.
- [Learn Next.js](https://nextjs.org/learn) - an interactive Next.js tutorial.

You can check out [the Next.js GitHub repository](https://github.com/vercel/next.js) - your feedback and contributions are welcome!

## Deploy on Vercel

The easiest way to deploy your Next.js app is to use the [Vercel Platform](https://vercel.com/new?utm_medium=default-template&filter=next.js&utm_source=create-next-app&utm_campaign=create-next-app-readme) from the creators of Next.js.

Check out our [Next.js deployment documentation](https://nextjs.org/docs/app/building-your-application/deploying) for more details.

# === Introduction ===

In this lab, we will explore AI-driven development tools and their applications in building modern web applications. You will learn about the following tools:

- **Generative AI**: The broader category of AI technologies that includes tools like ChatGPT, Gemini, and Claude.
- **V0**: A tool for rapid UI prototyping and component generation.
- **Cursor**: An AI-powered IDE for coding and debugging.
- **ChatGPT API**: A conversational AI API for natural language processing.
- **GitHub Copilot**: An AI coding assistant for accelerating development.

By the end of this lab, you will build a simple Next.js chat application that integrates the ChatGPT API, with UI components created using V0 and all of this logic implemented using Cursor as IDE.

# What is Generative AI?

Generative AI refers to artificial intelligence systems that can generate new content, such as text, images, videos, or audio, based on patterns they’ve learned from existing data. Unlike traditional AI models that classify or predict based on input data, generative AI creates new and unique outputs.

### Core Capabilities

- **Text Generation**: Writing human-like text for applications like chatbots, content creation, and summarization.
- **Image and Video Generation**: Creating visuals with tools like DALL-E and Stable Diffusion.
- **Audio Generation**: Composing music or synthesizing realistic human speech.

### Applications of Generative AI

- **Chatbots and Virtual Assistants**: Used in customer service and personal assistants (e.g., ChatGPT, Alexa).
- **Content Creation**: Writing articles, generating designs, or producing marketing materials.
- **Creative Tools**: Assisting artists, musicians, and designers in generating innovative ideas and assets.

## ChatGPT

**Overview**

ChatGPT is one of the most well-known examples of generative AI, developed by OpenAI. It uses large language models like GPT-3.5 and GPT-4 to understand and generate human-like text. ChatGPT is versatile and can handle tasks ranging from answering questions to writing essays or generating code.

## Key Features

- **Human-Like Conversations**: Engages in meaningful and context-aware discussions.
- **Flexibility**: Supports various use cases, including education, entertainment, and business.
- **Customization**: Can be tailored to specific tasks with system prompts and fine-tuning.

## Gemini

**Overview**

Gemini is Google's generative AI model, designed to combine advanced language understanding with cutting-edge capabilities in vision and multimodal reasoning. Released as part of Google DeepMind's AI ecosystem, Gemini aims to bring generative AI into various professional and creative domains.

### Key Features:

- **Multimodal Capabilities**: Understands and generates content across text, images, and other formats.
- **Enterprise Integration**: Tailored for applications in business, including document analysis and automation.
- **Advanced Context Understanding**: Excels in maintaining context across complex interactions.

## Claude

**Overview**

Claude is a conversational AI developed by Anthropic, designed with a focus on safety, ethical behavior, and robust reasoning. It competes with ChatGPT in providing conversational and creative AI solutions.

### Key Features

- **Safety-Oriented Design**: Built with principles to minimize harmful outputs and prioritize ethical usage.
- **Context Handling**: Handles long and complex conversations effectively.
- **Business Applications**: Supports automation, knowledge retrieval, and customer engagement.

## **Comparison of ChatGPT, Gemini, and Claude:**

| Feature           | ChatGPT                          | Gemini                           | Claude                          |
| ----------------- | -------------------------------- | -------------------------------- | ------------------------------- |
| Developer         | OpenAI                           | Google DeepMind                  | Anthropic                       |
| Key Strength      | Versatility in text generation   | Multimodal capabilities          | Safety and ethical alignment    |
| Target Audience   | General-purpose, developers      | Businesses, creators             | Businesses, ethical users       |
| Notable Use Cases | Chatbots, education, programming | Image-text applications, content | Customer service, knowledge ops |

# V0

### Overview

V0 is a visual development platform that enables rapid prototyping and development of user interfaces. It allows developers and designers to create UI components visually, reducing the need for manual coding of UI elements.

### History

V0 emerged in the landscape of low-code/no-code development tools, which have gained popularity in recent years to bridge the gap between design and development. The goal of such tools is to accelerate the development process and make it accessible to a wider audience, including those with limited coding experience.

### Key Benefits

- **Rapid Prototyping**: V0 allows for quick creation of user interfaces using a visual editor, which speeds up the design and iteration process.
- **Collaboration**: Designers and developers can work together seamlessly, as the tool provides a common platform for UI development.
- **Code Export**: V0 can export code in various formats (e.g., React components), which can be integrated into existing projects.

### Usage in the Lab

In this lab, you will use V0 to design the frontend of your chat application, creating components like the chat window, input field, and send button. This will allow you to focus on functionality without spending excessive time on UI coding.

# Cursor

### Overview

Cursor is an AI-powered Integrated Development Environment (IDE) designed to enhance developer productivity through intelligent code suggestions and assistance. Cursor uses machine learning models to provide context-aware code completions, refactoring suggestions, and error detection.

### History

The development of Cursor is part of the broader trend of integrating AI into software development tools. With advances in machine learning and natural language processing, tools like Cursor aim to augment the capabilities of developers, allowing them to write code more efficiently and with fewer errors.

### Key Features

- **AI-Assisted Coding**: Provides real-time code suggestions based on the context of your current work.
- **Error Detection and Debugging**: Helps identify potential issues in your code before runtime.
- **Productivity Boost**: Automates repetitive tasks and reduces the cognitive load on developers.

### Usage in the Lab

You will use Cursor to implement your chat application. The AI assistance will help you write efficient code.

# GitHub Copilot

### Overview

GitHub Copilot is an AI-powered code completion tool developed by GitHub in collaboration with OpenAI. It uses OpenAI's Codex model, which is trained on a vast dataset of public code repositories, to provide code suggestions and generate code snippets based on comments and existing code.

### History

Released in technical preview in June 2021, GitHub Copilot represents a significant advancement in AI-assisted programming. It aims to assist developers by suggesting code snippets and functions, thereby reducing the amount of boilerplate code they need to write manually.

### Key Features

- **Intelligent Code Suggestions**: Generates code based on context, comments, and function names.
- **Support for Multiple Languages**: Works with various programming languages, including Python, JavaScript, TypeScript, Ruby, and Go.
- **Learning from Context**: Adapts to the coding style and patterns of the developer over time.

### Usage in the Lab

While GitHub Copilot is not a primary tool in this lab, it can be used to assist with coding tasks, providing suggestions and speeding up development as you work on your project.

# ChatGPT API

### Overview

ChatGPT API provides developers with access to OpenAI's powerful language models, such as GPT-3.5 and GPT-4, enabling the integration of natural language understanding and generation into applications.

### History

OpenAI released GPT-3 in June 2020, which was followed by the development of ChatGPT, a model fine-tuned for conversational interactions. The ChatGPT API was made available to developers to harness the capabilities of these models in their own applications, allowing for a wide range of use cases, from chatbots to content generation.

### Key Features

- **Natural Language Understanding**: Capable of interpreting and processing human language input.
- **Conversational Responses**: Generates coherent and contextually appropriate responses in a conversational format.
- **Customization**: Developers can guide the model's behavior using system prompts and parameters.

### Usage in the Lab

You will integrate the ChatGPT API into your chat application, allowing it to generate AI-driven responses to user inputs.

## Tasks

- [ ] Download the generated project se-lab8-tasks.zip (and run `npm install` and `npm start`)
- [ ] Use V0 to design the frontend of your chat application.
- [ ] Implement the chat logic with help from Cursor / V0 / Chat GPT or other tools.
- [ ] Integrate the ChatGPT API to generate responses.
- [ ] Add a privacy policy/terms of service to your application using a generative AI tool.
- **Bonus**: Use AI generated images for your chatbot avatar.

<note tip>
  hint: [Leonardo.AI](https://leonardo.ai/), [DALL-E](https://openai.com/dall-e-2/), [Stable Diffusion](https://stability.ai/stable-diffusion)
</note>
